@@ -10,10 +10,11 @@
  */
 
 #include "des.h"
+#include <xmemory.h>
 
-#ifndef XMEMORY_H
-# define XMEMORY_H
-# include <xmemory.h>
+#ifndef SWAP_H
+# define SWAP_H
+# include <swap.h>
 #endif
 
 /* #define	NULL	0 */
@@ -161,7 +162,7 @@ static char p32i[] = {
 /* End of DES-defined tables */
 
 /* Lookup tables initialized once only at startup by desinit() */
-static word32 (*sp)[64];	/* Combined S and P boxes */
+static word32 (*sp)[64];		/* Combined S and P boxes */
 
 static char (*iperm)[16][8];	/* Initial and final permutations */
 static char (*fperm)[16][8];
