@@ -1,5 +1,4 @@
-/* Copyright (C) 1991, 1992, 1993 Free Software Foundation, Inc.
- *
+/* Copyright (C) 1998,1999 Nikos Mavroyanopoulos
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
@@ -32,12 +31,14 @@
 #include <tean.h>
 #include <twofish.h>
 #include <xmemory.h>
+#include <rc2.h>
 
 #define MAX_THREADS 256 /* After that pointers are being reused */
 
 /* I think now is re-entrant */
 
 static int check_pointer[MAX_THREADS]={0};
+static int used_all_threads=0;
 
 #ifdef HAVE_LIBPTHREAD
 # include <pthread.h>
@@ -56,6 +57,7 @@ static char **kn1=NULL; /* For DES */
 static char **kn2=NULL;
 static char **kn3=NULL; /* For tripleDES */
 static word8** saferkey=NULL; /* For safer64 and 128 */
+static word16 **rc2_key; /* For rc2 */
 
 /* For CBC mode */
 static word32 **previous_ciphertext;
@@ -68,6 +70,8 @@ static word8 **s_register;
 static word8 **enc_s_register;
 static word8 **enc_sd_register;
 
+
+
 /* hold the key and the algorithm */
 static word8 **keyword_given=NULL;
 static int *algorithm_given=NULL;
@@ -84,6 +88,7 @@ int get_block_size(int algorithm)
    case SAFER128:
    case CAST128:
    case TEAN:
+   case RC2:
       return 8;
       break;
 
@@ -119,6 +124,10 @@ int get_key_size(int algorithm) /* In bytes */
       return 56; 
       break;
 
+   case RC2:
+      return 128;
+      break;
+
    case GOST:
       return 32;
       break;
@@ -151,6 +160,7 @@ char *keyword1, *keyword2, *keyword3; /* For 3DES and safer*/
 unsigned char *skey=NULL;
 word8 twokey[16]; /* 128 bits */
 
+ if (used_all_threads==0) {
   saferkey=mxrealloc(saferkey, (thread+1)*sizeof(char*));
   kn1=mxrealloc(kn1, (thread+1)*sizeof(char*));
   kn2=mxrealloc(kn2, (thread+1)*sizeof(char*));
@@ -159,9 +169,10 @@ word8 twokey[16]; /* 128 bits */
   CT=mxrealloc(CT, (thread+1) * sizeof(TWOFISH_context));
   ckey=mxrealloc(ckey, (thread+1) * sizeof(struct cast_key));
   keyword_given=mxrealloc(keyword_given, (thread+1)*sizeof(char*));
+  rc2_key=mxrealloc(rc2_key, (thread+1)*sizeof(word16*));
   algorithm_given=mxrealloc(algorithm_given, (thread+1)*sizeof(int));
-  
-  
+ }
+  rc2_key[thread]=malloc(64*sizeof(word16));  
   if (get_key_size(algorithm) < lenofkey) lenofkey=get_key_size(algorithm);
 
   check_pointer[thread]=1;  
@@ -196,6 +207,11 @@ word8 twokey[16]; /* 128 bits */
       return thread;
       break; 
 
+   case RC2:
+      rc2_keyschedule( rc2_key[thread], (void*)keyword_given[thread], lenofkey, 1024);
+      return thread;
+      break; 
+
    case SAFER128:
       Safer_Init_Module();
       keyword1=mxmalloc(8); keyword2=mxmalloc(8);
@@ -290,6 +306,11 @@ int mcrypt(int thread, void *plaintext)
       cast_encrypt(&ckey[thread], (void*) plaintext);
       return 0;
       break;
+
+   case RC2:
+      rc2_encrypt( rc2_key[thread], (void*) plaintext, (void*) plaintext);
+      return 0;
+      break;
             
    case DES:
       endes ( (void*)kn1[thread], plaintext);
@@ -351,6 +372,12 @@ int mdecrypt(int thread, void *plaintext)
       return 0;
       break;
 
+   case RC2:
+
+      rc2_decrypt( rc2_key[thread], (void*) plaintext, (void*) plaintext);
+      return 0;
+      break;
+
    case DES:
       dedes ( (char (*)[8])kn1[thread], plaintext);
       return 0;
@@ -413,6 +440,10 @@ check_pointer[td]=0;
 	free(kn2[td]);
 	free(kn3[td]);
       break;
+
+   case RC2:
+   	free(rc2_key[td]);
+      break;
       
    case BLOWFISH:
    case TEAN:   
@@ -420,9 +451,9 @@ check_pointer[td]=0;
 
    case GOST:
    case ThreeWAY:
-	break;
    case CAST128:
         break;
+        
    case SAFER64:
    case SAFER128:
 	free(saferkey[td]);   
@@ -448,6 +479,7 @@ pthread_mutex_lock(&counter_cur_thread);
 
   if (cur_thread == MAX_THREADS) {
      cur_thread=0;
+     used_all_threads=1;
   }  
 
   while (check_pointer[cur_thread]==1 && cur_thread < MAX_THREADS) {
@@ -541,14 +573,17 @@ cur_thread++;
 if (cur_thread < MAX_THREADS && check_pointer[cur_thread]==0) {
 
 /* for cbc */
+ if (used_all_threads==0) {
   previous_ciphertext=mxrealloc(previous_ciphertext, (cur_thread+1) * sizeof(word32*));
   previous_cipher=mxrealloc(previous_cipher, (cur_thread+1) * sizeof(word32*));
   previous_plaintext=mxrealloc(previous_plaintext, (cur_thread+1) * sizeof(word32*));
+ }
 /* end cbc */
 
 }
 else {
   cur_thread=0;
+  used_all_threads=1;
   while (check_pointer[cur_thread]==1 && cur_thread <= MAX_THREADS) {
    cur_thread++;
   }  
@@ -667,14 +702,17 @@ cur_thread++;
 if (cur_thread < MAX_THREADS && check_pointer[cur_thread]==0) {
 
 /* for cfb */
+ if (used_all_threads==0) {
   s_register=mxrealloc(s_register, (cur_thread+1) * sizeof(char*));
   enc_s_register=mxrealloc(enc_s_register, (cur_thread+1) * sizeof(char*));  
   enc_sd_register=mxrealloc(enc_sd_register, (cur_thread+1) * sizeof(char*));  
   sd_register=mxrealloc(sd_register, (cur_thread+1) * sizeof(char*));
+ }
 /* end cfb */
 }
 else {
   cur_thread=0;
+  used_all_threads=1;
 
   while (check_pointer[cur_thread]==1 && cur_thread <= MAX_THREADS) {
    cur_thread++;
@@ -796,14 +834,17 @@ cur_thread++;
 if (cur_thread < MAX_THREADS && check_pointer[cur_thread]==0) {
 
 /* for ofb */
+ if (used_all_threads==0) {
   s_register=mxrealloc(s_register, (cur_thread+1) * sizeof(char*));
   enc_s_register=mxrealloc(enc_s_register, (cur_thread+1) * sizeof(char*));  
   enc_sd_register=mxrealloc(enc_sd_register, (cur_thread+1) * sizeof(char*));  
   sd_register=mxrealloc(sd_register, (cur_thread+1) * sizeof(char*));
+ }
 /* end ofb */
 }
 else {
   cur_thread=0;
+  used_all_threads=1;
 
   while (check_pointer[cur_thread]==1 && cur_thread <= MAX_THREADS) {
    cur_thread++;
@@ -901,13 +942,17 @@ int mdecrypt_ofb(int td, void *plaintext,int len) /* plaintext is 1 byte (8bit c
 
 char* get_algorithms_name(int algorithm) {
 
-char *x=mxmalloc(12);
+char *x=mxmalloc(20);
 
   switch(algorithm) {
    case DES:
     strcpy(x,"DES");
     return x;
     break;
+   case RC2:
+    strcpy(x,"RC2");
+    return x;
+    break;
    case TripleDES:
     strcpy(x,"3DES");
     return x;
@@ -916,12 +961,20 @@ char *x=mxmalloc(12);
     strcpy(x,"3-WAY");
     return x;
     break;
+
+#ifdef MCRYPT2
+   case CRYPT:
+    strcpy(x,"Crypt");
+    return x;
+    break;
+#endif
+
    case BLOWFISH:
-    strcpy(x,"BLOWFISH");
+    strcpy(x,"Blowfish");
     return x;
     break;
    case TWOFISH:
-    strcpy(x,"TWOFISH");
+    strcpy(x,"Twofish");
     return x;
     break;    
    case GOST:
