@@ -22,6 +22,8 @@
 # define ZIP
 #endif
 
+
+
 #ifdef HAVE_SIGNAL_H
 # include <signal.h>
 #endif
@@ -37,12 +39,15 @@
 # include <utime.h>
 #endif
 
-
 /* for open */
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
 
+#ifdef HAVE_SYS_WAIT_H
+# include <sys/wait.h>
+#endif
+
 #ifdef HAVE_PWD_H
 # include <pwd.h>
 #endif
@@ -74,11 +79,11 @@
 #define CFB 2
 #define OFB 3
 
+#define MAX_ALGO 11
+
 #define ENCRYPT 0
 #define DECRYPT 1
 
-#define CRYPT 5
-
 #if SIZEOF_UNSIGNED_LONG_INT == 4
  typedef unsigned long word32;
  typedef signed long sword32;
