@@ -21,6 +21,8 @@
 # include <defines.h>
 #endif
 
+#define MAX_KEY_LEN 300
+
 #include <lcrypt.h>
 #include <bzero.h>
 #include "extra.h"
@@ -37,13 +39,13 @@ extern int cleanDelete;
 
 char * my_getpass(char * prt) {
 char *atmp;
-char ztmp[128];
+char ztmp[MAX_KEY_LEN];
 char *btmp;
 
    atmp=getpass(prt);
 
    Bzero(ztmp,sizeof(ztmp));
-   strncat(ztmp,atmp,127);
+   strncat(ztmp,atmp,MAX_KEY_LEN-1);
 
    btmp=getpass(_("Re-Enter key: "));
    
@@ -97,12 +99,6 @@ int show_mode(int ende, int stype,int smode,char *output,int bitmode)
 {
 char *x=get_algorithms_name(stype);
 
-
- if (stype==CRYPT) {
-  x=malloc(12);
-  strcpy(x,"Unix Crypt");
- }
-
  fprintf (stderr,_( "Algorithm: %s\n"),x);
  free(x); 
  
@@ -305,7 +301,7 @@ void cleanUp() {
 char** read_key_file(char * file, int* num) {
 
   FILE* FROMF;
-  char keyword[128], **keys=NULL;
+  char keyword[MAX_KEY_LEN], **keys=NULL;
   int x=0;
 
   FROMF=fopen(file,"r");
@@ -320,7 +316,7 @@ char** read_key_file(char * file, int* num) {
   }
 
 
-  while (fgets( keyword, 128, FROMF)!=NULL) {
+  while (fgets( keyword, MAX_KEY_LEN, FROMF)!=NULL) {
     x++;
     keys=realloc(keys, x*sizeof(char*));
     keys[x-1]=malloc(strlen(keyword)+1);
