@@ -48,6 +48,7 @@
  unsigned int mode=CBC; /* 0:cbc 1:ecb */
  unsigned int type=BLOWFISH; /* default: blowfish */
  int quiet=TRUE; /* silent by default */
+ int tmpi;
  int unlink_flag=FALSE;
  int sunlink_flag=FALSE; 
  int bare_flag=FALSE, real_random_flag=FALSE;
@@ -61,20 +62,24 @@
 void usage(char *name)
 {
 
-     fprintf(stderr,_("Usage: mcrypt [-dLFus87bhvrx] [-f keyfile] [-k key1 key2 ...] [-m mode] [-i bitmode] [-a algorithm] [-c config_file] [file ...]\n\n"));
+     fprintf(stdout,_("Usage: mcrypt [-dLFus87bhvrx] [-f keyfile] [-k key1 key2 ...] [-m mode] [-i bitmode] [-a algorithm] [-c config_file] [file ...]\n\n"));
 
-     fprintf(stderr,_("mini-crypt encrypts/decrypts files using several symmetric block algorithms.\n"));
-     fprintf(stderr,_("Legal parameters are:\n\
+     fprintf(stdout,_("mini-crypt encrypts/decrypts files using several symmetric block algorithms.\n"));
+     fprintf(stdout,_("Legal parameters are:\n\
 \n\
  -d --decrypt                 decrypts\n\
  -c --config FILENAME         Use the configuration file FILENAME.\n\
                               If not specified the .mcryptrc in your home\n\
                               directory is used.\n\
  -a --algorithm ALGORITHM     specify the encryption and decryption algorithm.\n\
-                              Legal parameters are: 3way, des, 3des, gost, tea,\n\
-                              safer64, safer128, cast128, crypt, twofish\n\
-                              and blowfish. (Default is blowfish)\n\
- -m --mode MODE               specify the mode (cfb, ofb, ecb or cbc).\n\
+                              Legal parameters are:\n"));
+
+   for (tmpi=0; tmpi<(MAX_ALGO+1); tmpi++) {
+    fprintf(stdout, "                              - %s\n", get_algorithms_name(tmpi));
+   }
+    fprintf(stdout, _("                              (Default is blowfish)\n"));
+                              
+    fprintf(stdout,_(" -m --mode MODE               specify the mode (cfb, ofb, ecb or cbc).\n\
                               (Default is cbc)\n\
  -k --key KEY1, KEY2, ...     keyword(s) to use.\n\
  -f --keyfile FILENAME        file to read the keyword from.\n\
@@ -91,18 +96,18 @@ void usage(char *name)
  -F --force                   forces output to stdout.\n"));
 
 #ifdef HAVE_DEV_RANDOM
-fprintf(stderr,_(" -r --random                  Use real random data.\n"));
+fprintf(stdout,_(" -r --random                  Use real random data.\n"));
 #endif
 
 #ifdef GZIP
-fprintf(stderr,_(" -z --gzip                    use gzip to compress files before encryption.\n"));
+fprintf(stdout,_(" -z --gzip                    use gzip to compress files before encryption.\n"));
 #endif
 
 #ifdef BZIP2
-fprintf(stderr,_(" -p --bzip2                   use bzip2 to compress files before encryption.\n"));
+fprintf(stdout,_(" -p --bzip2                   use bzip2 to compress files before encryption.\n"));
 #endif
 
-fprintf(stderr,_("\n\
+fprintf(stdout,_("\n\
  -h --help                    prints this help\n\
  -v --version                 prints the version number\n\
 \n\
@@ -110,7 +115,7 @@ Report bugs to mcrypt@i-net.paiko.gr.\n\n"));
 }
 
 void mcrypt_license() {
-  fprintf (stderr,_("\nCopyright (C) 1998,1999 Nikos Mavroyanopoulos\n\
+  fprintf (stdout,_("\nCopyright (C) 1998,1999 Nikos Mavroyanopoulos\n\
 This program is free software; you can redistribute it and/or modify \n\
 it under the terms of the GNU General Public License as published by \n\
 the Free Software Foundation; either version 2 of the License, or \n\
@@ -368,11 +373,11 @@ int main(int argc,char **argv)
                               type=CRYPT;
                               }
                               else {
-                                if (strncasecmp(typet,"SAFER64",7)==0) {
+                                if (strncasecmp(typet,"SAFER64",7)==0 || strncasecmp(typet,"SAFER-SK64",10)==0) {
                                   type=SAFER64;
                                 }
                                 else {
-                                  if (strncasecmp(typet,"SAFER128",8)==0) {
+                                  if (strncasecmp(typet,"SAFER128",8 || strncasecmp(typet,"SAFER-SK128",11)==0)==0) {
                                     type=SAFER128;
                                   }
                                   else {
@@ -380,18 +385,23 @@ int main(int argc,char **argv)
                                     type=CAST128;
                                     }
                                     else {
-                                       if (strncasecmp(typet,"TEA",3)==0) {
+                                       if (strncasecmp(typet,"TEA",3)==0 || strncasecmp(typet,"xTEA",4)==0) {
                                         type=TEAN;
                                        }
                                        else { 
                                           if (strncasecmp(typet,"TWOFISH",7)==0) {
                                            type=TWOFISH;
                                           }
-                                          else { /* Default */
-                                           type=BLOWFISH;
-                                           sprintf(tmperr, _("No such algorithm: %s\n"),typet);
-                                           err_crit (tmperr);
-                                           err_crit (_("Assuming Blowfish.\n"));
+                                          else {
+                                              if (strncasecmp(typet,"RC2",3)==0) {
+                                               type=RC2;
+                                              }                                          
+                                              else { /* Default */
+                                                type=BLOWFISH;
+                                                 sprintf(tmperr, _("No such algorithm: %s\n"),typet);
+                                                 err_crit (tmperr);
+                                                 err_crit (_("Assuming Blowfish.\n"));
+                                              }
                                           }
                                        }
                                     }
@@ -608,6 +618,7 @@ for(i=0;i<file_count;i++) {
 				case ThreeWAY:
 				case BLOWFISH:
                                 case TWOFISH:
+                                case RC2:
 				case GOST:
 				case SAFER64:
 				case SAFER128:
@@ -689,6 +700,7 @@ for(i=0;i<file_count;i++) {
 	        	case ThreeWAY:
 	        	case BLOWFISH:
                         case TWOFISH:
+                        case RC2:
 	        	case GOST:
 	        	case SAFER64:
 	        	case SAFER128:
@@ -824,7 +836,16 @@ char* command=NULL;
         strcat(command, fromfile);
 
     	FROMF=popen(command ,"r");
-	if (FROMF==NULL) {perror("fopen");return 1;}
+      /* Waitpid is called here because popen does not return
+         NULL if the execution of the program fails.
+         thanx to Vasilis Vasaitis <vvas@hal.csd.auth.gr>
+       */
+
+#ifdef HAVE_WAITPID
+	if (FROMF==NULL || waitpid(-1, NULL, WNOHANG)==0) {perror("popen");return 1;}
+#else
+	if (FROMF==NULL) {perror("popen");return 1;}
+#endif	
 	
 	TOF=fopen(tofile,"wb");
 	if (TOF==NULL) {perror("fopen");return -1;}
@@ -893,10 +914,15 @@ while(ch==0);
    fflush(FROMF);
    fflush(TOF);
    unlock( fileno((FILE*)TOF));
-   unlock( fileno((FILE*)FROMF));
+   if (gzipflag==FALSE && bzipflag==FALSE) unlock( fileno((FILE*)FROMF));
  }
    fclose(TOF);
-   fclose(FROMF);
+   if (gzipflag==FALSE && bzipflag==FALSE) {
+     fclose(FROMF);
+   }
+   else {
+     pclose(FROMF);
+   }
    
    Bzero(keyword, sizeof(keyword));
    
@@ -982,8 +1008,11 @@ if (mode==CBC || mode==CFB || mode==OFB) {
           strcat(command, fromfile);
 
       	  FROMF=popen(command ,"r");
-  	  if (FROMF==NULL) {perror("fopen");return 1;}
-	
+#ifdef HAVE_WAITPID
+  	  if (FROMF==NULL || waitpid(-1, NULL, WNOHANG)==0) {perror("popen");return 1;}
+#else
+  	  if (FROMF==NULL) {perror("popen");return 1;}	
+#endif  	  
 	  RTOF=fopen(tofile,"wb");
 	  if (RTOF==NULL) {perror("fopen");return -1;}
 	  if (write_lock( fileno((FILE*)TOF))==-1) return -1;
@@ -1248,13 +1277,18 @@ switch (mode) {
  if (stream_flag==FALSE) {
     fflush(FROMF);
     fflush(RTOF);    
-    unlock( fileno((FILE*)FROMF) );
+    if (gzipflag==FALSE && bzipflag==FALSE) unlock( fileno((FILE*)FROMF) );
     unlock( fileno((FILE*)RTOF) );
  }
                 
     fclose(TOF);
     fclose(RTOF);
-    fclose(FROMF);
+    if (gzipflag==FALSE && bzipflag==FALSE) {
+      fclose(FROMF);
+    }
+    else {
+      pclose(FROMF);
+    }
 /* Ready */
 
 
@@ -1635,11 +1669,19 @@ switch (mode) {
     if (stream_flag==FALSE) {
      if (gzipflag==TRUE) {
        err_warn(_("Decompressing the output file...\n"));
-       execlp(GZIP, GZIP, "-d", tofile, NULL);
+       i=execlp(GZIP, GZIP, "-d", tofile, NULL);
+       if (i==-1) {
+         perror("exec");
+         return 1;
+       }
      }
      if (bzipflag==TRUE) {
        err_warn(_("Decompressing the output file...\n"));
-       execlp(BZIP2, BZIP2, "-d", tofile, NULL);
+       i=execlp(BZIP2, BZIP2, "-d", tofile, NULL);
+       if (i==-1) {
+         perror("exec");
+         return 1;
+       }       
      }
     }
 #endif
